unit CustomAttribute;

interface

type
  TMyCustomAttribute = class(TCustomAttribute)
  private
    FValue : Integer;
    FDescription: string;
  public
    constructor Create (aValue: Integer);
    function GetValue: Integer;
    property Description: string read FDescription write FDescription;
  end;

  procedure ShowCustomAttributes (aType: System.Type);

implementation

uses
  System.Reflection, SysUtils;

constructor TMyCustomAttribute.Create(aValue: Integer);
begin
  inherited Create;
  FValue := aValue;
end;

function TMyCustomAttribute.GetValue: Integer;
begin
  Result := FValue;
end;

procedure ShowAttribs (attribs: array of System.Object);
var
  attr: TMyCustomAttribute;
begin
  // show only the first one...
  if Length (attribs) > 0 then
  begin
    write (' ---> ' + attribs[0].ToString);
    attr := attribs[0] as TMyCustomAttribute;
    write ('(' + IntToStr (attr.GetValue) + ' -- ' + attr.Description + ')');
  end;
end;

procedure ShowCustomAttributes (aType: System.Type);
var
  members : array of System.Reflection.MemberInfo;
  I: Integer;
  mtypes: System.Reflection.MemberTypes;
begin
  write (aType.Name);
  ShowAttribs (aType.GetCustomAttributes
    (TMyCustomAttribute.ClassInfo, True));
  writeln;

  members := aType.GetMembers;
  for I := 0 to High(members) do
  begin
    mtypes := members[i].MemberType;
    write (aType.Name + ':' + members[i].Name +
      ' (' + TObject(mtypes).ToString + ')');
    ShowAttribs (members[i].GetCustomAttributes
      (TMyCustomAttribute.ClassInfo, True));
    writeln; // new line
  end;
end;

end.
